import * as React from "react";
import createEmotionServer from "@emotion/server/create-instance";
import createCache from "@emotion/cache";
import DefaultDocument from "next/document";
export default function augmentDocumentWithEmotionCache(options) {
    const { prepend, ...optionsWithoutPrependProp } = options;
    const appPropName = `${options.key}EmotionCache`;
    const insertionPointId = `${options.key}-emotion-cache-insertion-point`;
    return (Document) => {
        const super_getInitialProps = Document.getInitialProps?.bind(Document) ??
            DefaultDocument.getInitialProps.bind(DefaultDocument);
        Document.getInitialProps = async (documentContext) => {
            const cache = createCache(optionsWithoutPrependProp);
            const emotionServer = createEmotionServer(cache);
            const originalRenderPage = documentContext.renderPage;
            documentContext.renderPage = ({ enhanceApp, ...params }) => originalRenderPage({
                ...params,
                "enhanceApp": (App) => {
                    const EnhancedApp = enhanceApp?.(App) ?? App;
                    return function EnhanceApp(props) {
                        return (React.createElement(EnhancedApp, { ...{ ...props, [appPropName]: cache } }));
                    };
                }
            });
            const initialProps = await super_getInitialProps(documentContext);
            const emotionStyles = [
                React.createElement("style", { id: insertionPointId, key: insertionPointId }),
                ...emotionServer
                    .extractCriticalToChunks(initialProps.html)
                    .styles.filter(({ css }) => css !== "")
                    .map(style => (React.createElement("style", { "data-emotion": `${style.key} ${style.ids.join(" ")}`, key: style.key, dangerouslySetInnerHTML: { "__html": style.css } })))
            ];
            const otherStyles = React.Children.toArray(initialProps.styles);
            return {
                ...initialProps,
                "styles": prepend
                    ? [...emotionStyles, ...otherStyles]
                    : [...otherStyles, ...emotionStyles]
            };
        };
    };
}
