import * as React from "react";
import createCache from "@emotion/cache";
import { CacheProvider as DefaultCacheProvider } from "@emotion/react";
import { assert } from "../tools/assert";
export default function withAppEmotionCache(options, CacheProvider = DefaultCacheProvider) {
    return (App) => {
        const { ...optionsWithoutPrependProp } = options;
        const appPropName = `${options.key}EmotionCache`;
        const insertionPointId = `${options.key}-emotion-cache-insertion-point`;
        const createClientSideCache = (() => {
            let cache = undefined;
            return () => {
                if (cache !== undefined) {
                    return cache;
                }
                return (cache = createCache({
                    ...optionsWithoutPrependProp,
                    "insertionPoint": (() => {
                        // NOTE: Under normal circumstances we are on the client.
                        // It might not be the case though, see: https://github.com/garronej/tss-react/issues/124
                        const isBrowser = typeof document === "object" &&
                            typeof document?.getElementById === "function";
                        if (!isBrowser) {
                            return undefined;
                        }
                        const htmlElement = document.getElementById(insertionPointId);
                        assert(htmlElement !== null);
                        return htmlElement;
                    })()
                }));
            };
        })();
        function AppWithEmotionCache(props) {
            const { [appPropName]: cache, ...rest } = props;
            return (React.createElement(CacheProvider, { value: cache ?? createClientSideCache() },
                React.createElement(App, { ...rest })));
        }
        Object.keys(App).forEach(staticMethod => (AppWithEmotionCache[staticMethod] = App[staticMethod]));
        AppWithEmotionCache.displayName = AppWithEmotionCache.name;
        return AppWithEmotionCache;
    };
}
